/*
 * This file handles all raw GPIO output.
 */

#ifndef GPIO_HANDLER_C
#define GPIO_HANDLER_C

#include <linux/gpio.h>
#include <linux/printk.h>

/* Sets all GPIO pin levels to low. */
void sevenseg_gpio_clear(void)
{
    gpio_set_value(GPIO_TOP,          GPIO_LOW);
    gpio_set_value(GPIO_TOP_LEFT,     GPIO_LOW);
    gpio_set_value(GPIO_TOP_RIGHT,    GPIO_LOW);
    gpio_set_value(GPIO_MIDDLE,       GPIO_LOW);
    gpio_set_value(GPIO_BOTTOM_LEFT,  GPIO_LOW);
    gpio_set_value(GPIO_BOTTOM_RIGHT, GPIO_LOW);
    gpio_set_value(GPIO_BOTTOM,       GPIO_LOW);
    gpio_set_value(GPIO_DOT,          GPIO_LOW);
}

/* Writes the specified numbers to the GPIO pins. */
void sevenseg_gpio_display(const struct number number)
{
    gpio_set_value(GPIO_TOP,          number.top);
    gpio_set_value(GPIO_TOP_LEFT,     number.top_left);
    gpio_set_value(GPIO_TOP_RIGHT,    number.top_right);
    gpio_set_value(GPIO_MIDDLE,       number.middle);
    gpio_set_value(GPIO_BOTTOM_LEFT,  number.bottom_left);
    gpio_set_value(GPIO_BOTTOM_RIGHT, number.bottom_right);
    gpio_set_value(GPIO_BOTTOM,       number.bottom);
    gpio_set_value(GPIO_DOT,          number.dot);

    // Debug (GPIO is high off so we have to inverse the bit for better visualization)
    printk(KERN_DEBUG "sevenseg: Display image:");
    printk(KERN_DEBUG "sevenseg:  %i%i%i\n", inverse(number.top), inverse(number.top), inverse(number.top));

    printk(KERN_DEBUG "sevenseg: %i   %i\n", inverse(number.top_left), inverse(number.top_right));
    printk(KERN_DEBUG "sevenseg: %i   %i\n", inverse(number.top_left), inverse(number.top_right));

    printk(KERN_DEBUG "sevenseg:  %i%i%i\n", inverse(number.middle), inverse(number.middle), inverse(number.middle));

    printk(KERN_DEBUG "sevenseg: %i   %i\n", inverse(number.bottom_left), inverse(number.bottom_right));
    printk(KERN_DEBUG "sevenseg: %i   %i\n", inverse(number.bottom_left), inverse(number.bottom_right));

    printk(KERN_DEBUG "sevenseg:  %i%i%i %i\n", inverse(number.bottom), inverse(number.bottom), inverse(number.bottom), inverse(number.dot));
}

/* Called when a new number from udev has been received to update the display. */
void sevenseg_gpio_update(unsigned int number)
{
    if (number > 0xF)
    {
        printk(KERN_DEBUG "sevenseg: Received number 0x%x, turning display off", number);
        sevenseg_gpio_clear();
        return;
    }
    printk(KERN_DEBUG "sevenseg: Update received, new number: 0x%x\n", number);
    sevenseg_gpio_display(numbers[number]);
}

/* Called when the kernel module is being initialized */
int sevenseg_gpio_init(void)
{
    int err;
    printk(KERN_DEBUG "sevenseg: Initializing GPIO pins.\n");
    
    //TODO make dis shit work
    err = gpio_direction_output(GPIO_TOP,          GPIO_LOW);
    if(err) return err;

    err = gpio_direction_output(GPIO_TOP_LEFT,     GPIO_LOW);
    if(err) return err;

    err = gpio_direction_output(GPIO_TOP_RIGHT,    GPIO_LOW);
    if(err) return err;

    err = gpio_direction_output(GPIO_MIDDLE,       GPIO_LOW);
    if(err) return err;

    err = gpio_direction_output(GPIO_BOTTOM_LEFT,  GPIO_LOW);
    if(err) return err;

    err = gpio_direction_output(GPIO_BOTTOM_RIGHT, GPIO_LOW);
    if(err) return err;

    err = gpio_direction_output(GPIO_BOTTOM,       GPIO_LOW);
    if(err) return err;

    err = gpio_direction_output(GPIO_DOT,          GPIO_LOW);
    
    return err;
}

/* Called when the kernel module is exited */
void sevenseg_gpio_exit(void)
{
    printk(KERN_DEBUG "sevenseg: Releasing GPIO pins.\n");
    sevenseg_gpio_clear();

    gpio_free(GPIO_TOP);
    gpio_free(GPIO_TOP_LEFT);
    gpio_free(GPIO_TOP_RIGHT);
    gpio_free(GPIO_MIDDLE);
    gpio_free(GPIO_BOTTOM_LEFT);
    gpio_free(GPIO_BOTTOM_RIGHT);
    gpio_free(GPIO_BOTTOM);
    gpio_free(GPIO_DOT);
}

#endif