/*
 * This header file provides some utility functions like converting characters to integers.
 */

#ifndef UTILS_H
#define UTILS_H

/*
 * Converts a single character to a hex integer.
 * Possible values are (case-insensitive): 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, A, B, C, D, E, F
 * 
 * @param c The character to be converted.
 * @return The character as an integer; 0x10 if the character is NaN
 */
unsigned int char_to_int(char c)
{
    switch(c)
    {
        case '0': return 0x0;
        case '1': return 0x1;
        case '2': return 0x2;
        case '3': return 0x3;
        case '4': return 0x4;
        case '5': return 0x5;
        case '6': return 0x6;
        case '7': return 0x7;
        case '8': return 0x8;
        case '9': return 0x9;
        case 'a':
        case 'A': return 0xA;
        case 'b':
        case 'B': return 0xB;
        case 'c':
        case 'C': return 0xC;
        case 'd':
        case 'D': return 0xD;
        case 'e':
        case 'E': return 0xE;
        case 'f':
        case 'F': return 0xF;

        default:  return 0x10;
    }
};

/* Inverts the boolean value of an integer. */
int inverse(int i)
{
    return i ^ 1;
}

struct number
{
    int top;
    int top_left;
    int top_right;
    int middle;
    int bottom_left;
    int bottom_right;
    int bottom;
    int dot;
} extern const numbers[16];

const struct number numbers[16] = {
//   TOP        TOP LEFT   TOP RIGHT  MIDDLE     BOTTOM L   BOTTOM R   BOTTOM     DOT            DISPLAY NUMBER
    {GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_LOW }, // 0x0
    {GPIO_LOW,  GPIO_LOW,  GPIO_HIGH, GPIO_LOW,  GPIO_LOW,  GPIO_HIGH, GPIO_LOW,  GPIO_LOW }, // 0x1
    {GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_HIGH}, // 0x2
    {GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_HIGH, GPIO_HIGH}, // 0x3
    {GPIO_LOW,  GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_LOW,  GPIO_LOW }, // 0x4
    {GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_HIGH, GPIO_HIGH}, // 0x5
    {GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH}, // 0x6
    {GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_LOW,  GPIO_LOW,  GPIO_HIGH, GPIO_LOW,  GPIO_LOW }, // 0x7
    {GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_LOW }, // 0x8
    {GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_HIGH, GPIO_HIGH}, // 0x9
    {GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_LOW }, // 0xA
    {GPIO_LOW,  GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_LOW }, // 0xB
    {GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_LOW,  GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_LOW }, // 0xC
    {GPIO_LOW,  GPIO_LOW,  GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_HIGH, GPIO_LOW }, // 0xD
    {GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_HIGH}, // 0xE
    {GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_HIGH, GPIO_HIGH, GPIO_LOW,  GPIO_LOW,  GPIO_LOW }  // 0xF
};


#endif