/*
 * This program is a linux kernel module that controls a seven segment display
 * connected to a Raspberry Pi's GPIO pins.
 * 
 * This module has been designed and tested for the Raspberry Pi v3 revision B.
 * It is possible that it works with other hardware as well, however, there
 * won't be any support.
 * 
 * For the documentation visit https://mygit.th-deg.de/fk16790/SevenSegController/
 * 
 * Copyright (c) 2017 Felix Kopp    <felix.kopp@stud.th-deg.de>
 *                and Armin Weigold <armin.weigold@stud.th-deg.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef SEVENSEG_C
#define SEVENSEG_C

#include <linux/init.h>
#include <linux/module.h>

// Include the other source code files
#include <include/autoinclude.h>

/* Main method for initializing the kernel module */
static int __init sevenseg_init(void)
{
    int error;
    
    printk(KERN_INFO "sevenseg: Initializing...\n");

    // Initialize the GPIO ports
    error = sevenseg_gpio_init();
    if(error)
    {
        printk(KERN_ERR "sevenseg: Error: GPIO initialization failed. Exiting.\n");
        return error;
    }

    // Initialize the udev interface
    error = sevenseg_udev_init();
    if(error)
    {
        // Print a warning to the kernel log and return the exit code
        printk(KERN_ERR "sevenseg: Could not rise the udev interface. Exiting.\n");
        sevenseg_gpio_exit();
        return error;
    }

    printk(KERN_INFO "sevenseg: The ting goes skrrrrraaah!\n");
    return 0;
}

/* Exit method to unload the module */
static void __exit sevenseg_exit(void)
{
    printk(KERN_INFO "sevenseg: Exiting...\n");
    
    sevenseg_udev_exit();
    sevenseg_gpio_exit();

    printk(KERN_INFO "sevenseg: Thank you and goodbye.\n");
}

// Report entry and exit point to the kernel
module_init(sevenseg_init);
module_exit(sevenseg_exit);

// License and author info
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Felix Kopp <felix.kopp@stud.th-deg.de>");
MODULE_AUTHOR("Armin Weigold <armin.weigold@stud.th-deg.de>");
MODULE_DESCRIPTION("Seven segment display driver");
MODULE_SUPPORTED_DEVICE("sevenseg");
MODULE_VERSION("0.1");

#endif