#!/bin/bash

# This script test the functionality of the seven segment display kernel module.
# Return values are:
# 0: Test completed without errors
# 1: Kernel module not loaded
# 2: Kernel module loaded, but device file /dev/sevenseg does not exist
# 3: The script was not run as superuser

values=('0' '1' '2' '3' '4' '5' '6' '7' '8' '9' 'A' 'B' 'C' 'D' 'E' 'F' ' ')
device="/dev/sevenseg"

echo "Checking if module is loaded..."

if [[ $(whoami) != 'root' ]];
then
    
    echo "You are not root! Exiting."
    exit 3
    
fi

if [[ $(cat /proc/modules | grep "sevenseg") ]];
then
    
    if [ -c "$device" ];
    then
        
        echo "Module loaded correctly and character device $device exists. Testing..."
        
        for i in "${values[@]}";
        do
            sleep 1
            echo $i > $device
            echo "echo '$i' > $device"
        done
        
        echo "Test completed."
        exit 0
        
    else
        
        echo "Module is loaded, but character device $device does not exist."
        echo "Please check if you loaded the correct module or try reloading it. Exiting."
        exit 2
        
    fi

else
    
    echo "Module is not loaded, exiting."
    exit 1
    
fi
